@@ -3,11 +3,20 @@ function(libname, pkgname)
 {
  library.dynam("XML", pkgname, libname)
 
- if(exists("setMethod")) {
+ if(.useNamespacesInXMLPackage && exists("setMethod")) {
    .InitSAXMethods()
  }
 
 }
+
+.onLoad =
+function(...)
+{
+ if(exists("setMethod")) {
+   .InitSAXMethods()
+ }
+}
+
 #
 #  Copyright (c) 1998, 1999 The Omega Project for Statistical Computing.
 #       All rights reserved.#
